DEFINT A-Z

'$STATIC

'Structure Definitions
' *Thanks to Ali for these info

TYPE Vector
     x AS SINGLE
     y AS SINGLE
     z AS SINGLE
END TYPE        '12 bytes

TYPE UV
     x AS SINGLE
     y AS SINGLE
END TYPE        '8 bytes

TYPE short4
     No1 AS INTEGER
     No2 AS INTEGER
     No3 AS INTEGER
     No4 AS INTEGER
END TYPE        '8 bytes

TYPE vertcolor4
     No1 AS STRING * 4
     No2 AS STRING * 4
     No3 AS STRING * 4
     No4 AS STRING * 4
END TYPE        '16 bytes

TYPE TexPos4
     No1 AS UV
     No2 AS UV
     No3 AS UV
     No4 AS UV
END TYPE        '32 bytes

TYPE PrmHeader
     PolyNum AS INTEGER
     VertNum AS INTEGER
END TYPE        '4 bytes

TYPE Polygon
     PolyType  AS INTEGER               '2
     Texture   AS INTEGER               '2
     VertIndex AS short4                '8
     VertColor AS vertcolor4            '16
     TexCoord  AS TexPos4               '32

END TYPE        '60 bytes

TYPE Vertex
     Position AS Vector
     normal AS Vector
END TYPE        '24 bytes

DIM InPolygon AS Polygon, OutPolygon AS Polygon
DIM InPrm AS PrmHeader, InVertex AS Vertex, OutVertex AS Vertex

CONST PolyStructSize = 60
CONST VertStructSize = 24
CONST PrmHeaderSize = 4

Main:
 ArgNo% = 1
 PRINT "This program will reverse Re-Volt model."

'This section is unsure to work...Qbasic cannot pass command line.
'So don't use command line arguments, please input the file name after run.
Arguments:
 IF ArgNo% > 9 GOTO error4
 Argument$ = ENVIRON$(LEFT$(CHR$(ArgNo%), 1)): ArgNo% = ArgNo% + 1
 IF Argument$ = "" THEN GOSUB AskInputFileName ELSE InputFileName$ = Argument$
 IF LEFT$(InputFileName$, 1) = "/" GOTO Switches
 IF LCASE$(RIGHT$(InputFileName$, 4)) <> ".prm" AND LCASE$(RIGHT$(InputFileName$, 2)) <> ".m" GOTO error1

 InputFile = FREEFILE
 
 OPEN InputFileName$ FOR INPUT AS InputFile: CLOSE InputFile
 OPEN InputFileName$ FOR BINARY AS InputFile

 OutputFileName$ = ENVIRON$(CHR$(ArgNo%))
 IF OutputFileName$ = "" AND Argument$ <> "" THEN
    OutputFileName$ = Argument$
 
  ELSEIF OutputFileName$ = "" THEN
    GOSUB AskOutputFileName
    IF OutputFileName$ = "" THEN OutputFileName$ = InputFileName$
    
 END IF
 IF OutputFileName$ = InputFileName$ THEN OutputFile = InputFile ELSE OutputFile = FREEFILE

'Check the number of polygons and vertices
 GET InputFile, 1, InPrm
 IF InPrm.PolyNum = 0 GOTO error2
 
 IF OutputFile <> InputFile THEN
    OPEN OutputFileName$ FOR OUTPUT AS OutputFile: CLOSE OutputFile
    OPEN OutputFileName$ FOR BINARY AS OutputFile
    PUT OutputFile, 1, InPrm
 END IF

 PRINT "# of polygons:"; InPrm.PolyNum
 PRINT "# of vertices:"; InPrm.VertNum

' Reverse polygons

 FOR i% = 0 TO InPrm.PolyNum - 1
     GET InputFile, 1 + PrmHeaderSize + PolyStructSize * i%, InPolygon
     OutPolygon = InPolygon

     IF (InPolygon.PolyType AND 1) = 1 THEN     'If polygon is square
        OutPolygon.VertIndex.No1 = InPolygon.VertIndex.No4
        OutPolygon.VertColor.No1 = InPolygon.VertColor.No4
         OutPolygon.TexCoord.No1 = InPolygon.TexCoord.No4
        OutPolygon.VertIndex.No2 = InPolygon.VertIndex.No3
        OutPolygon.VertColor.No2 = InPolygon.VertColor.No3
         OutPolygon.TexCoord.No2 = InPolygon.TexCoord.No3
        OutPolygon.VertIndex.No3 = InPolygon.VertIndex.No2
        OutPolygon.VertColor.No3 = InPolygon.VertColor.No2
         OutPolygon.TexCoord.No3 = InPolygon.TexCoord.No2
        OutPolygon.VertIndex.No4 = InPolygon.VertIndex.No1
        OutPolygon.VertColor.No4 = InPolygon.VertColor.No1
         OutPolygon.TexCoord.No4 = InPolygon.TexCoord.No1

      ELSE                                      'If polygon is triangle
        OutPolygon.VertIndex.No1 = InPolygon.VertIndex.No3
        OutPolygon.VertColor.No1 = InPolygon.VertColor.No3
         OutPolygon.TexCoord.No1 = InPolygon.TexCoord.No3
        OutPolygon.VertIndex.No2 = InPolygon.VertIndex.No2
        OutPolygon.VertColor.No2 = InPolygon.VertColor.No2
         OutPolygon.TexCoord.No2 = InPolygon.TexCoord.No2
        OutPolygon.VertIndex.No3 = InPolygon.VertIndex.No1
        OutPolygon.VertColor.No3 = InPolygon.VertColor.No1
         OutPolygon.TexCoord.No3 = InPolygon.TexCoord.No1
        OutPolygon.VertIndex.No4 = InPolygon.VertIndex.No4
        OutPolygon.VertColor.No4 = InPolygon.VertColor.No4
         OutPolygon.TexCoord.No4 = InPolygon.TexCoord.No4
     END IF
    
     PUT OutputFile, 1 + PrmHeaderSize + PolyStructSize * i%, OutPolygon
 NEXT

'Copy vertices

 Offset% = 1 + PrmHeaderSize + PolyStructSize * InPrm.PolyNum
 FOR i% = 0 TO InPrm.VertNum - 1
     GET InputFile, Offset% + VertStructSize * i%, InVertex
     OutVertex.Position = InVertex.Position
     OutVertex.normal.x = -InVertex.normal.x
     OutVertex.normal.y = -InVertex.normal.y
     OutVertex.normal.z = -InVertex.normal.z

     PUT OutputFile, Offset% + VertStructSize * i%, OutVertex
 NEXT
 
 CLOSE
 PRINT "Reversed model saved to "; OutputFileName$; "."
 GOTO finish

'Ask file name
AskInputFileName:
 LINE INPUT "Input file name[*.prm/*.m]:", InputFileName$
 RETURN
AskOutputFileName:
 PRINT "Output file name["; InputFileName$; "]:";
 LINE INPUT "", OutputFileName$
 RETURN

error1:
 PRINT "Only .prm or .m file supported"
 GOTO finish

error2:
 PRINT "No polygon found"
 GOTO finish

error3:
 PRINT "Unknown switch"

error4:
 PRINT "Too many arguments"

help:
 PRINT "This program reverses polygons of Re-Volt models(.prm or .m file)."
 GOTO finish

finish:
 END

'Switch recognising

Switches:
 SELECT CASE arg$
    
     CASE "/?"
          GOTO help
    
     CASE ELSE
          GOTO error3

 END SELECT
 GOTO Arguments

